@@ -4,5 +4,5 @@ __all__ = ("ConsumeParams",)
 
 
 class ConsumeParams(JavaClass, metaclass=MetaJavaClass):
-    __javaclass__ = f"com/android/billingclient/api/ConsumeParams"
+    __javaclass__ = "com/android/billingclient/api/ConsumeParams"
     newBuilder = JavaStaticMethod("()Lcom/android/billingclient/api/ConsumeParams$Builder;")
